@@ -6,7 +6,7 @@ import time
 import datetime
 import urllib
 import webbrowser
-import commands
+import subprocess
 import re
 import sys
 import traceback
@@ -26,7 +26,7 @@ import mimetools, mimetypes
 import os, stat
 
 from Nagstamon import Objects
-from Nagstamon.Objects import Node, Result
+from Nagstamon.Objects import Result
     
 #from Nagstamon import GUI
 import GUI
@@ -85,7 +85,8 @@ class RefreshLoopOneServer(threading.Thread):
               
         while self.stopped == False:          
             # check if we have to leave update interval sleep
-            if self.server.count > int(self.conf.update_interval)*60: self.doRefresh = True       
+            if self.server.count > int(self.conf.update_interval)*60: self.doRefresh = True   
+
             # self.doRefresh could also been changed by RefreshAllServers()
             if self.doRefresh == True:              
                 # reset server count
@@ -96,12 +97,12 @@ class RefreshLoopOneServer(threading.Thread):
                     self.server.status = "Refreshing"
                     gobject.idle_add(self.output.popwin.UpdateStatus, self.server)
                     # get current status
-                    server_status = self.server.GetStatus()
+                    server_status = self.server.GetStatus(output=self.output)
                     # GTK/Pango does not like tag brackets < and >, so clean them out from description
                     server_status.error = server_status.error.replace("<", "").replace(">", "").replace("\n", " ")
                     # debug
                     if str(self.conf.debug_mode) == "True":
-                        self.server.Debug(server=self.server.get_name(), debug="server return values: " + server_status.result + " " + server_status.error)
+                        self.server.Debug(server=self.server.get_name(), debug="server return values: " + str(server_status.result) + " " + str(server_status.error))
                     if server_status.error != "":
                         # set server status for status field in popwin 
                         self.server.status = "ERROR"
@@ -113,6 +114,8 @@ class RefreshLoopOneServer(threading.Thread):
                         # of lost network connectivity - this leads to a mysterious pango crash
                         if self.output.statusbar.isShowingError == False:
                             gobject.idle_add(self.output.RefreshDisplayStatus)
+                            ###time.sleep(0.05)
+                            ###gobject.idle_add(self.output.RefreshDisplayStatus)
                             # wait a moment
                             time.sleep(5)
                             # change statusbar to the following error message
@@ -130,10 +133,11 @@ class RefreshLoopOneServer(threading.Thread):
                         self.server.status = "Connected"
                         # tell gobject to care about GUI stuff - refresh display status
                         gobject.idle_add(self.output.RefreshDisplayStatus)
+                        ###time.sleep(0.05)
+                        ###gobject.idle_add(self.output.RefreshDisplayStatus)
                         # wait for the doRefresh flag to be True, if it is, do a refresh
                         if self.doRefresh == True:
                             if str(self.conf.debug_mode) == "True":
-                                #print self.server.get_name(), ":", "Refreshing output - server is already checking:", self.server.isChecking                                        
                                 self.server.Debug(server=self.server.get_name(), debug="Refreshing output - server is already checking: " + str(self.server.isChecking))
                             # reset refresh flag
                             self.doRefresh = False
@@ -156,7 +160,6 @@ def RefreshAllServers(servers=None, output=None, conf=None):
         if server.isChecking == False and str(conf.servers[server.get_name()].enabled) == "True":
             #debug
             if str(conf.debug_mode) == "True":
-                #print "Checking server:", server.get_name()
                 server.Debug(server=server.get_name(), debug="Checking server...")
     
             server.thread.Refresh()
@@ -196,7 +199,7 @@ class DebugLoop(threading.Thread):
                 self.debug_file = open(self.conf.debug_file, "w")
             except Exception, err:
                 # if path to file does not exist tell user
-                self.output.ErrorDialog(err) 
+                self.output.Dialog(message=err) 
         
         
     def run(self):       
@@ -262,10 +265,9 @@ class RecheckAll(threading.Thread):
             try:
                 # debug
                 if str(self.conf.debug_mode) == "True":
-                    #print "Recheck all: Rechecking all services on all hosts on all servers..."
                     # workaround, take Debug method from first server reachable
                     self.servers.values()[0].Debug(debug="Recheck all: Rechecking all services on all hosts on all servers...")
-                for server in self.servers.values():                
+                for server in self.servers.values():      
                     # only test enabled servers and only if not already 
                     if str(self.conf.servers[server.get_name()].enabled):
                         # set server status for status field in popwin
@@ -278,7 +280,6 @@ class RecheckAll(threading.Thread):
                             rechecks_dict[server.get_name() + ": " + host.get_name()].start()
                             # debug
                             if str(self.conf.debug_mode) == "True":
-                                #print "Recheck all:", "rechecking", server.get_name() + ": " + host.get_name()                                
                                 server.Debug(server=server.get_name(), host=host.get_name(), debug="Rechecking...")
                             for service in host.services.values():
                                 # dito
@@ -289,12 +290,11 @@ class RecheckAll(threading.Thread):
                                 # debug
                                 if str(self.conf.debug_mode) == "True":
                                     server.Debug(server=server.get_name(), host=host.get_name(), service=service.get_name(), debug="Rechecking...")
-
+        
                 # wait until all rechecks have been done
                 while len(rechecks_dict) > 0:
                     # debug
                     if str(self.conf.debug_mode) == "True":
-                        #print "Recheck all: # of checks which still need to be done:", len(rechecks_dict)
                         # once again taking .Debug() from first server
                         self.servers.values()[0].Debug(server=server.get_name(), debug="Recheck all: # of checks which still need to be done: " + str(len(rechecks_dict)))
 
@@ -307,7 +307,6 @@ class RecheckAll(threading.Thread):
                     
                 # debug
                 if str(self.conf.debug_mode) == "True":
-                    #print "Recheck all: All servers, hosts and services are rechecked."
                     # once again taking .Debug() from first server
                     self.servers.values()[0].Debug(server=server.get_name(), debug="Recheck all: All servers, hosts and services are rechecked.")                
                 # reset global flag
@@ -325,7 +324,6 @@ class RecheckAll(threading.Thread):
         else:
             # debug
             if str(self.conf.debug_mode) == "True":
-                #print "Recheck all: Already rechecking all services on all hosts on all servers."
                 # once again taking .Debug() from first server
                 self.servers.values()[0].Debug(debug="Recheck all: Already rechecking all services on all hosts on all servers.")                
 
@@ -362,6 +360,20 @@ def Downtime_get_start_end(server, host):
     # get start and end time from Nagios as HTML - the objectified HTML does not contain the form elements :-(
     # this used to happen in GUI.action_downtime_dialog_show but for a more strict separation it better stays here
     return server.get_start_end(host)
+
+
+class SubmitCheckResult(threading.Thread):
+    """
+    exceute remote cgi command with parameters from submit check result dialog 
+    """
+    def __init__(self, **kwds):
+        # add all keywords to object, every mode searchs inside for its favorite arguments/keywords
+        for k in kwds: self.__dict__[k] = kwds[k]
+        threading.Thread.__init__(self)
+        self.setDaemon(1)
+
+    def run(self):
+        self.server.set_submit_check_result(self)
         
 
 class CheckForNewVersion(threading.Thread):
@@ -381,7 +393,6 @@ class CheckForNewVersion(threading.Thread):
         
         # debug
         if str(self.output.conf.debug_mode) == "True":
-            #print "Checking for new version..."
             # once again taking .Debug() from first server
             self.servers.values()[0].Debug(debug="Checking for new version...")
 
@@ -391,13 +402,14 @@ class CheckForNewVersion(threading.Thread):
             if s.CheckingForNewVersion == False:
                 # set the flag to lock that connection
                 s.CheckingForNewVersion = True
+                # use IFW server to speed up request and secure via https
+                result = s.FetchURL("https://nagstamon.ifw-dresden.de/files-nagstamon/latest_version_" +\
+                                     self.output.version, giveback="raw", no_auth=True)
                 # remove newline
-                result = s.FetchURL("http://nagstamon.sourceforge.net/latest_version", giveback="raw")
                 version, error = result.result.split("\n")[0], result.error
                 
                 # debug
                 if str(self.output.conf.debug_mode) == "True":
-                    #print "Latest version from sourceforge.net:", version
                     # once again taking .Debug() from first server
                     self.servers.values()[0].Debug(debug="Latest version from sourceforge.net: " + str(version))
                 
@@ -448,6 +460,7 @@ class PlaySound(threading.Thread):
                 self.Play(self.conf.notification_custom_sound_down)
         elif self.sound =="FILE":
             self.Play(self.file)
+            
     
     def Play(self, file):
         """
@@ -456,14 +469,14 @@ class PlaySound(threading.Thread):
         # debug
         if str(self.conf.debug_mode) == "True":
             # once again taking .Debug() from first server
-            self.servers.values()[0].Debug(debug="Playing sound: " + file)
+            self.servers.values()[0].Debug(debug="Playing sound: " + str(file))
         if not platform.system() == "Windows":
-            commands.getoutput("play -q %s" % file)
+            subprocess.Popen("play -q %s" % str(file), shell=True)
         else:
             winsound.PlaySound(file, winsound.SND_FILENAME)
             
                     
-class FlashStatusbar(threading.Thread):
+class Notification(threading.Thread):
     """
         Flash statusbar in a threadified way to omit hanging gui
     """
@@ -475,52 +488,147 @@ class FlashStatusbar(threading.Thread):
 
 
     def run(self):
-        # in case of notifying in statusbar do some flashing
-        if self.output.Notifying == True:
+        # counter for repeated sound
+        soundcount = 0
+        # in case of notifying in statusbar do some flashing and honking
+        while self.output.Notifying == True:
             # as long as flashing flag is set statusbar flashes until someone takes care
-            while self.output.statusbar.Flashing == True:
+            if self.output.statusbar.Flashing == True:
                 if self.output.statusbar.isShowingError == False:
                     # check again because in the mean time this flag could have been changed by NotificationOff()
                     gobject.idle_add(self.output.statusbar.Flash)
-                    time.sleep(0.5)
+            # if wanted play notification sound, if it should be repeated every minute (2*interval/0.5=interval) do so.
+            if str(self.conf.notification_sound) == "True":
+                if soundcount == 0:
+                    sound = PlaySound(sound=self.sound, Resources=self.Resources, conf=self.conf, servers=self.servers)
+                    sound.start()
+                    soundcount += 1
+                elif str(self.conf.notification_sound_repeat) == "True" and soundcount >= 2*int(self.conf.update_interval)*60:
+                    soundcount = 0
+                else:
+                    soundcount += 1       
+            time.sleep(0.5)
         # reset statusbar
         self.output.statusbar.Label.set_markup(self.output.statusbar.statusbar_labeltext)
+        
+
+class MoveStatusbar(threading.Thread):
+    """
+        Move statusbar in a threadified way to omit hanging gui and Windows-GTK 2.22 trouble
+    """
+    def __init__(self, **kwds):
+        # add all keywords to object, every mode searchs inside for its favorite arguments/keywords
+        for k in kwds: self.__dict__[k] = kwds[k]
+        threading.Thread.__init__(self)
+        self.setDaemon(1)
 
 
-def OpenNagios(widget, server, output):   
-    # open Nagios main page in your favorite web browser when nagios button is clicked
-    # first close popwin
-    output.popwin.Close()
-    # start browser with URL
-    server.open_nagios()
-
-
-def OpenServices(widget, server, output):
-    # open Nagios services in your favorite web browser when service button is clicked
-    # first close popwin
-    output.popwin.Close()
-    # start browser with URL
-    server.open_services()
- 
-   
-def OpenHosts(widget, server, output):
-    # open Nagios hosts in your favorite web browser when hosts button is clicked
-    # first close popwin
-    output.popwin.Close()
-    # start browser with URL
-    server.open_hosts()
-
+    def run(self):
+        # avoid flickering popwin while moving statusbar around
+        # gets re-enabled from popwin.setShowable()
+        if self.output.GUILock.has_key("Popwin"): self.output.popwin.Close()
+        self.output.popwin.showPopwin = False
+        # lock GUI while moving statusbar so no auth dialogs could pop up
+        self.output.AddGUILock(self.__class__.__name__)
+        # in case of moving statusbar do some moves
+        while self.output.statusbar.Moving == True:
+            gobject.idle_add(self.output.statusbar.Move)
+            time.sleep(0.01)
+        self.output.DeleteGUILock(self.__class__.__name__)
+        
+            
+class Action(threading.Thread):
+    """
+    Exectute custom actions triggered by context menu of popwin
+    parameters are action and hosts/service
+    """
+    def __init__(self, **kwds):
+        # add all keywords to object
+        self.host = ""
+        self.service = ""
+        for k in kwds: self.__dict__[k] = kwds[k]
+        threading.Thread.__init__(self)
+        self.setDaemon(1)
+        
+        
+    def run(self):
+        # first replace placeholder variables in string with actual values
+        """
+        Possible values for variables:
+        $HOST$             - host as in monitor
+        $SERVICE$          - service as in monitor
+        $MONITOR$          - monitor address - not yet clear what exactly for
+        $MONITOR-CGI$      - monitor CGI address - not yet clear what exactly for
+        $ADDRESS$          - address of host, investigated by Server.GetHost()
+        $USERNAME$         - username on monitor
+        $PASSWORD$         - username's password on monitor - whatever for
+        $COMMENT-ACK$      - default acknowledge comment
+        $COMMENT-DOWN$     - default downtime comment
+        $COMMENT-SUBMIT$   - default submit check result comment
+        """
+        try:
+            # if run as custom action use given action definition
+            if self.__dict__.has_key("action"):
+                string = self.action.string
+                action_type = self.action.type
+            else:
+                string = self.string
+                action_type = self.type
+                
+            # mapping of variables and values
+            mapping = { "$HOST$": self.host,\
+                        "$SERVICE$": self.service,\
+                        "$ADDRESS$": self.server.GetHost(self.host).result,\
+                        "$MONITOR$": self.server.monitor_url,\
+                        "$MONITOR-CGI$": self.server.monitor_cgi_url,\
+                        "$USERNAME$": self.server.username,\
+                        "$PASSWORD$": self.server.password,\
+                        "$COMMENT-ACK$": self.conf.defaults_acknowledge_comment,\
+                        "$COMMENT-DOWN$": self.conf.defaults_downtime_comment,\
+                        "$COMMENT-SUBMIT$": self.conf.defaults_submit_check_result_comment,
+                        }
+            # mapping mapping
+            for i in mapping:
+                string = string.replace(i, mapping[i])
+            
+            # see what action to take
+            if action_type == "browser":
+                # make string ready for URL
+                string = self._URLify(string)
+                # debug
+                if str(self.conf.debug_mode) == "True":
+                    self.server.Debug(server=self.server.name, host=self.host, service=self.service, debug="ACTION: Browser " + string)
+                webbrowser.open(string)
+            elif self.action.type == "url":
+                # make string ready for URL
+                string = self._URLify(string)
+                # debug
+                if str(self.conf.debug_mode) == "True":
+                    self.server.Debug(server=self.server.name, host=self.host, service=self.service, debug="ACTION: URL in background " + string)
+                self.server.FetchURL(string)
+            elif self.action.type == "command":
+                # debug
+                if str(self.conf.debug_mode) == "True":
+                    self.server.Debug(server=self.server.name, host=self.host, service=self.service, debug="ACTION: COMMAND " + string)
+                subprocess.Popen(string, shell=True)
+        except:
+            import traceback
+            traceback.print_exc(file=sys.stdout)
+        
+            
+    def _URLify(self, string):
+        """
+        return a string that fulfills requirements for URL
+        exclude several chars
+        """
+        return urllib.quote(string, ":/=?&@")
+    
     
 def TreeViewNagios(server, host, service):
     # if the clicked row does not contain a service it mus be a host, 
     # so the nagios query is different 
     server.open_tree_view(host, service)
 
-        
-def TreeViewHTTP(host):
-    # open Browser with URL of some Host
-    webbrowser.open("http://" + host)
-
 
 # contains dict with available server classes
 # key is type of server, value is server class
@@ -548,7 +656,7 @@ def get_registered_server_type_list():
     return [x[0] for x in REGISTERED_SERVERS]
 
 
-def CreateServer(server=None, conf=None, debug_queue=None):
+def CreateServer(server=None, conf=None, debug_queue=None, resources=None):   
     # create Server from config
     registered_servers = get_registered_servers()
     if server.type not in registered_servers:
@@ -557,32 +665,29 @@ def CreateServer(server=None, conf=None, debug_queue=None):
     # give argument servername so CentreonServer could use it for initializing MD5 cache
     nagiosserver = registered_servers[server.type](conf=conf, name=server.name)
     nagiosserver.type = server.type
-    nagiosserver.nagios_url = server.nagios_url
-    nagiosserver.nagios_cgi_url = server.nagios_cgi_url
+    nagiosserver.monitor_url = server.monitor_url
+    nagiosserver.monitor_cgi_url = server.monitor_cgi_url           
+    # add resources, needed for auth dialog
+    nagiosserver.Resources = resources
     nagiosserver.username = server.username
-    if server.save_password or not server.enabled:
-        nagiosserver.password = server.password
-    else:
-        pwdialog = GUI.PasswordDialog(
-            "Password for " + server.username + " on " + server.nagios_url + ": ")
-        if pwdialog.password == None:
-            nagiosserver.password = ""
-        else:
-            nagiosserver.password = pwdialog.password        
+    nagiosserver.password = server.password 
     nagiosserver.use_proxy = server.use_proxy
     nagiosserver.use_proxy_from_os = server.use_proxy_from_os
     nagiosserver.proxy_address = server.proxy_address
     nagiosserver.proxy_username = server.proxy_username
     nagiosserver.proxy_password = server.proxy_password
     
+    # if password is not to be saved ask for it at startup
+    if str(server.save_password) == False: nagiosserver.refresh_authentication = True
+    
     # access to thread-safe debug queue
     nagiosserver.debug_queue = debug_queue     
     
     # use server-owned attributes instead of redefining them with every request
     nagiosserver.passman = urllib2.HTTPPasswordMgrWithDefaultRealm()
-    nagiosserver.passman.add_password(None, server.nagios_url, server.username, server.password)
-    nagiosserver.passman.add_password(None, server.nagios_cgi_url, server.username, server.password)  
-    nagiosserver.auth_handler = urllib2.HTTPBasicAuthHandler(nagiosserver.passman)
+    nagiosserver.passman.add_password(None, server.monitor_url, server.username, server.password)
+    nagiosserver.passman.add_password(None, server.monitor_cgi_url, server.username, server.password)  
+    nagiosserver.basic_handler = urllib2.HTTPBasicAuthHandler(nagiosserver.passman)
     nagiosserver.digest_handler = urllib2.HTTPDigestAuthHandler(nagiosserver.passman)  
     nagiosserver.proxy_auth_handler = urllib2.ProxyBasicAuthHandler(nagiosserver.passman)    
     
@@ -598,7 +703,7 @@ def CreateServer(server=None, conf=None, debug_queue=None):
     # create permanent urlopener for server to avoid memory leak with millions of openers    
     nagiosserver.urlopener = BuildURLOpener(nagiosserver)        
     # server's individual preparations for HTTP connections (for example cookie creation)
-    if server.enabled == True:
+    if str(server.enabled) == "True":
         nagiosserver.init_HTTP()    
 
     # debug
@@ -608,6 +713,11 @@ def CreateServer(server=None, conf=None, debug_queue=None):
     return nagiosserver
 
 
+def not_empty(x):
+    '''tiny helper function for BeautifulSoup in GenericServer.py to filter text elements'''
+    return bool(x.replace('&nbsp;', '').strip())
+
+
 def BuildURLOpener(server):
     """
     if there should be no proxy used use an empty proxy_handler - only necessary in Windows,
@@ -615,20 +725,21 @@ def BuildURLOpener(server):
     In UNIX $HTTP_PROXY will be used
     The MultipartPostHandler is needed for submitting multipart forms from Opsview
     """
-
+    # trying with changed digest/basic auth order as some digest auth servers do not
+    # seem to work wi the previous way
     if str(server.use_proxy) == "False":
         server.proxy_handler = urllib2.ProxyHandler({})
-        urlopener = urllib2.build_opener(server.auth_handler,\
-                                        server.digest_handler,\
-                                        server.proxy_handler,\
-                                        urllib2.HTTPCookieProcessor(server.Cookie),\
-                                        MultipartPostHandler)
+        urlopener = urllib2.build_opener(server.digest_handler,\
+                                         server.basic_handler,\
+                                         server.proxy_handler,\
+                                         urllib2.HTTPCookieProcessor(server.Cookie),\
+                                         MultipartPostHandler)
     elif str(server.use_proxy) == "True":
         if str(server.use_proxy_from_os) == "True":
-            urlopener = urllib2.build_opener(server.auth_handler,\
-                                            server.digest_handler,\
-                                            urllib2.HTTPCookieProcessor(server.Cookie),\
-                                            MultipartPostHandler)
+            urlopener = urllib2.build_opener(server.digest_handler,\
+                                             server.basic_handler,\
+                                             urllib2.HTTPCookieProcessor(server.Cookie),\
+                                             MultipartPostHandler)
         else:
             # if proxy from OS is not used there is to add a authenticated proxy handler
             server.passman.add_password(None, server.proxy_address, server.proxy_username, server.proxy_password)
@@ -636,8 +747,8 @@ def BuildURLOpener(server):
             server.proxy_auth_handler = urllib2.ProxyBasicAuthHandler(server.passman)
             urlopener = urllib2.build_opener(server.proxy_handler,\
                                             server.proxy_auth_handler,\
-                                            server.auth_handler,\
                                             server.digest_handler,\
+                                            server.basic_handler,\
                                             urllib2.HTTPCookieProcessor(server.Cookie),\
                                             MultipartPostHandler)
     return urlopener
@@ -653,53 +764,65 @@ def OpenNagstamonDownload(output=None):
     webbrowser.open("http://nagstamon.sourceforge.net/download")
        
 
+def IsFoundByRE(string, pattern, reverse):
+    """
+    helper for context menu actions in context menu - hosts and services might be filtered out
+    also useful for services and hosts and status information
+    """
+    pattern = re.compile(pattern)    
+    if len(pattern.findall(string)) > 0:
+        if str(reverse) == "True":
+            return False
+        else:
+            return True
+    else:
+        if str(reverse) == "True":
+            return True 
+        else:
+            return False    
+    
+    
 def HostIsFilteredOutByRE(host, conf=None):
     """
-        helper for applying RE filters in nagstamonGUI.RefreshDisplay()
+        helper for applying RE filters in Generic.GetStatus()
     """
     try:
         if str(conf.re_host_enabled) == "True":
-            pattern = re.compile(conf.re_host_pattern)           
-            if len(pattern.findall(host)) > 0:
-                if str(conf.re_host_reverse) == "True":
-                    return False
-                else:
-                    return True
-            else:
-                if str(conf.re_host_reverse) == "True":
-                    return True
-                else:
-                    return False
-        
+            return IsFoundByRE(host, conf.re_host_pattern, conf.re_host_reverse)
         # if RE are disabled return True because host is not filtered      
         return False
     except:
-        pass
+        import traceback
+        traceback.print_exc(file=sys.stdout)
         
         
 def ServiceIsFilteredOutByRE(service, conf=None):
     """
-        helper for applying RE filters in nagstamonGUI.RefreshDisplay()
+        helper for applying RE filters in Generic.GetStatus()
     """
     try:
         if str(conf.re_service_enabled) == "True":
-            pattern = re.compile(conf.re_service_pattern)           
-            if len(pattern.findall(service)) > 0:
-                if str(conf.re_service_reverse) == "True":
-                    return False
-                else:
-                    return True
-            else:
-                if str(conf.re_service_reverse) == "True":
-                    return True 
-                else:
-                    return False
-        
+            return IsFoundByRE(service, conf.re_service_pattern, conf.re_service_reverse)
         # if RE are disabled return True because host is not filtered      
         return False
     except:
-        pass
+        import traceback
+        traceback.print_exc(file=sys.stdout)
+
     
+def StatusInformationIsFilteredOutByRE(status_information, conf=None):
+    """
+        helper for applying RE filters in Generic.GetStatus()
+    """
+    try:
+        if str(conf.re_status_information_enabled) == "True":
+            return IsFoundByRE(status_information, conf.re_status_information_pattern, conf.re_status_information_reverse)
+        # if RE are disabled return True because host is not filtered      
+        return False
+    except:
+        import traceback
+        traceback.print_exc(file=sys.stdout)
+
 
 def HumanReadableDuration(seconds):
     """
@@ -726,9 +849,9 @@ def HumanReadableDuration(seconds):
         return seconds
 
     
-def MachineSortableDuration(raw):
+def MachineSortableDate(raw):
     """
-    Monitors gratefully shows duration even in weeks and months which confuse the
+    Monitors gratefully show duration even in weeks and months which confuse the
     sorting of popup window sorting - this functions wants to fix that
     """
     # dictionary for duration date string components
@@ -742,10 +865,57 @@ def MachineSortableDuration(raw):
     for c in raw.strip().replace("  ", " ").split(" "):
         number, period = c[0:-1],c[-1]
         d[period] = int(number) 
+        del number, period
     # convert collected duration data components into seconds for being comparable
     return 16934400 * d["M"] + 604800 * d["w"] + 86400 * d["d"] + 3600 * d["h"] + 60 * d["m"] + d["s"]
 
+
+def MachineSortableDateMultisite(raw):
+    """
+    Multisite dates/times are so different to the others so it has to be handled separately
+    """
+    # dictionary for duration date string components
+    d = {"M":0, "d":0, "h":0, "m":0, "s":0}
     
+    # if for some reason the value is empty/none make it compatible: 0 sec
+    if raw == None: raw = "0 sec"
+
+    # check_mk has different formats - if duration takes too long it changes its scheme
+    if "-" in raw and ":" in raw:
+        datepart, timepart = raw.split(" ")
+        # need to convert years into months for later comparison
+        Y, M, D = datepart.split("-")
+        d["M"] = int(Y) * 12 + int(M)
+        d["d"] = int(D)
+        # time does not need to be changed
+        h, m, s = timepart.split(":")
+        d["h"], d["m"], d["s"] = int(h), int(m), int(s)    
+        del datepart, timepart, Y, M, D, h, m, s
+    else:
+        # recalculate a timedelta of the given value 
+        if "sec" in raw:
+            d["s"] = raw.split(" ")[0]
+            delta = datetime.datetime.now() - datetime.timedelta(seconds=int(d["s"]))
+        elif "min" in raw:
+            d["m"] = raw.split(" ")[0]
+            delta = datetime.datetime.now() - datetime.timedelta(minutes=int(d["m"]))
+        elif "hrs" in raw:
+            d["h"] = raw.split(" ")[0]
+            delta = datetime.datetime.now() - datetime.timedelta(hours=int(d["h"])) 
+        else:
+            delta = datetime.datetime.now()
+        
+        Y, M, d["d"], d["h"], d["m"], d["s"] = delta.strftime("%Y %m %d %H %M %S").split(" ")
+        # need to convert years into months for later comparison
+        d["M"] = int(Y) * 12 + int(M)  
+            
+    # int-ify d
+    for i in d: d[i] = int(d[i])
+       
+    # convert collected duration data components into seconds for being comparable
+    return 16934400 * d["M"] + 86400 * d["d"] + 3600 * d["h"] + 60 * d["m"] + d["s"]
+
+
 def MD5ify(string):
     """
     makes something md5y of a given username or password for Centreon web interface access
@@ -753,54 +923,6 @@ def MD5ify(string):
     return md5(string).hexdigest()
 
 
-def ObjectifyXML(xmlraw):
-    """
-    replacement for lxml.objectify, until now only used for Centreon
-    """    
-    try:
-        nodes = []
-        for l in xmlraw.getchildren():
-            # only take l-nodes because they contain valuable information
-            # this is only valid for Centreon 
-            if l.tag == "l":
-                node = Node()
-                for e in l.getchildren():
-                    node.__dict__[e.tag] = node.add("text")
-                    node.__dict__[e.tag].text = e.text
-                nodes.append(node)
-        del xmlraw
-        return nodes
-    except:
-        del xmlraw
-        traceback.print_exc(sys.exc_info)
-        
-
-def ObjectifyHTML(htmlraw):
-    """
-    replacement for lxml.objectify, not really used yet
-    """    
-    try:
-        nodes = []
-        for l in htmlraw.getchildren():
-
-            print l
-            
-            # only take l-nodes because they contain valuable information
-            # this is only valid for Centreon 
-            if l.tag == "l":
-                node = Node()
-                for e in l.getchildren():
-                    node.__dict__[e.tag] = node.add("text")
-                    node.__dict__[e.tag].text = e.text
-                nodes.append(node)
-        del htmlraw
-        return nodes
-    except:
-        del htmlraw
-        traceback.print_exc(sys.exc_info)
-        
-        
-
 # <IMPORT>
 # Borrowed from http://pipe.scs.fsu.edu/PostHandler/MultipartPostHandler.py
 # Released under LGPL
