@@ -1,6 +1,6 @@
 # encoding: utf-8
 
-import urllib
+import urllib, urllib2
 import webbrowser
 import socket
 import time
@@ -8,20 +8,11 @@ import sys
 import cookielib
 import traceback
 import gc
-
-# new attempt to replace memory eating lxml
-try:
-    # from python 2.5 on it is in xml
-    from xml.etree import ElementTree
-except:
-    # older pythons will find it in elementtree
-    from elementtree import ElementTree
-
+   
 from Nagstamon import Actions
 from Nagstamon.Objects import *
 from Nagstamon.Server.Generic import GenericServer
 
-
 class CentreonServer(GenericServer): 
     TYPE = 'Centreon'
     # centreon generic web interface uses a sid which is needed to ask for news
@@ -29,68 +20,75 @@ class CentreonServer(GenericServer):
     # count for SID regeneration
     SIDcount = 0
     
+    # URLs for browser shortlinks/buttons on popup window
+    BROWSER_URLS= { "monitor": "$MONITOR$/main.php?p=1",\
+                    "hosts": "$MONITOR$/main.php?p=20103&o=hpb",\
+                    "services": "$MONITOR$/main.php?p=20202&o=svcpb",\
+                    "history": "$MONITOR$/main.php?p=203"}
+    
+    # A Monitor CGI URL is not necessary so hide it in settings
+    DISABLED_CONTROLS = ["label_monitor_cgi_url", "input_entry_monitor_cgi_url"]
+    
+    # newer Centreon versions (2.3+?) have different URL paths with a "/ndo" fragment
+    # will be checked by _get_ndo_url() but default is /xml/ndo/
+    XML_NDO = "xml/ndo"
+    
+    
     def __init__(self, **kwds):
         # add all keywords to object, every mode searchs inside for its favorite arguments/keywords
         for k in kwds: self.__dict__[k] = kwds[k]
 
         GenericServer.__init__(self, **kwds)
+        
+        # Entries for monitor default actions in context menu
+        self.MENU_ACTIONS = ["Monitor", "Recheck", "Acknowledge", "Downtime"] 
 
-        # cache MD5 username + password to reduce load
-        self.MD5_username = Actions.MD5ify(self.conf.servers[self.get_name()].username)   
-        self.MD5_password = Actions.MD5ify(self.conf.servers[self.get_name()].password)
-
+        
+    def init_HTTP(self):  
+        """
+        initialize HTTP connection
+        """
+        if self.HTTPheaders == {}:
+            GenericServer.init_HTTP(self)
+            # Centreon xml giveback method just should exist
+            self.HTTPheaders["xml"] = {}      
+            
+            
+    def reset_HTTP(self):
+        """
+        Centreon needs deletion of SID
+        """
+        self.HTTPheaders = {}     
+        self.SID = None
+        self.SIDcount = 0
+        self._get_sid()
+        
     
     def open_tree_view(self, host, service=""):
         # must be a host if service is empty...
         if service == "":
-            webbrowser.open(self.nagios_cgi_url + "/index.php?" + urllib.urlencode({"p":201, "autologin":1,\
-            "o":"hd", "useralias":self.MD5_username, "password":self.MD5_password, "host_name":host}))
+            webbrowser.open(self.monitor_cgi_url + "/main.php?" + urllib.urlencode({"p":201,"o":"hd", "host_name":host}))
         else:
-            webbrowser.open(self.nagios_cgi_url + "/index.php?" + urllib.urlencode({"p":202, "autologin":1,\
-            "o":"svcd", "useralias":self.MD5_username, "password":self.MD5_password, "host_name":host,\
-             "service_description":service}))       
+            webbrowser.open(self.monitor_cgi_url + "/main.php?" + urllib.urlencode({"p":202, "o":"svcd",  "host_name":host, "service_description":service}))      
 
             
-    def open_nagios(self):
-        webbrowser.open(self.nagios_cgi_url + "/index.php?autologin=1&p=1&useralias=" + self.MD5_username + "&password=" + self.MD5_password)
-        # debug
-        if str(self.conf.debug_mode) == "True":
-            self.Debug(server=self.get_name(), debug="Open monitor web page " + self.nagios_url + "/index.php?autologin=1&p=1&useralias=" + self.MD5_username + "&password=" + self.MD5_password)
-
-        
-    def open_services(self):
-        webbrowser.open(self.nagios_cgi_url + "/index.php?autologin=1&p=1&useralias=" + self.MD5_username + "&password=" + self.MD5_password + "&p=20202&o=svcpb")
-        # debug
-        if str(self.conf.debug_mode) == "True":
-            self.Debug(server=self.get_name(), debug="Open services web page " + "/index.php?p=20202&o=svcpb")
-
-            
-    def open_hosts(self):
-        webbrowser.open(self.nagios_cgi_url + "/index.php?autologin=1&p=1&useralias=" + self.MD5_username + "&password=" + self.MD5_password + "&p=20103&o=hpb")
-        # debug
-        if str(self.conf.debug_mode) == "True":
-            self.Debug(server=self.get_name(), debug="Open hosts web page " + self.nagios_url + "/index.php?autologin=1&p=1&useralias=" + self.MD5_username + "&password=" + self.MD5_password + "&p=20103&o=hpb")
-            
-            
     def get_start_end(self, host):
         """
         get start and end time for downtime from Centreon server
         """
         try:
-            cgi_data = urllib.urlencode({"p":"20305",\
+            cgi_data = urllib.urlencode({"p":"20106",\
                                          "o":"ah",\
                                          "host_name":host})
-            result = self.FetchURL(self.nagios_cgi_url + "/main.php?" + cgi_data, giveback="raw")
-            raw, error = result.result, result.error
-            if error == "":
-                # session id might have been invalid, so if necessary get a new one
-                if raw.find('name="start" type="text" value="') == -1:
-                    self.SID = self._get_sid().result
-                    result = self.FetchURL(self.nagios_cgi_url + "/main.php?" + cgi_data, giveback="raw")
-                    raw, error = result.result, result.error
-                start_time = raw.split('name="start" type="text" value="')[1].split('"')[0]
-                end_time = raw.split('name="end" type="text" value="')[1].split('"')[0]
-                del raw
+            #result = self.FetchURL(self.monitor_cgi_url + "/main.php?" + cgi_data, giveback="raw")
+            #raw, error = result.result, result.error
+            result = self.FetchURL(self.monitor_cgi_url + "/main.php?" + cgi_data, giveback="obj")
+            html, error = result.result, result.error
+            if error == "":               
+                html = result.result
+                start_time = html.find(attrs={"name":"start"}).attrMap["value"]
+                end_time = html.find(attrs={"name":"end"}).attrMap["value"]                  
+    
                 # give values back as tuple      
                 return start_time, end_time
         except:
@@ -103,6 +101,10 @@ class CentreonServer(GenericServer):
         Centreonified way to get host ip - attribute "a" in down hosts xml is of no use for up
         hosts so we need to get ip anyway from web page
         """
+        # the fastest method is taking hostname as used in monitor
+        if str(self.conf.connect_by_host) == "True" or host == "":
+            return Result(result=host)        
+        
         # do a web interface search limited to only one result - the hostname
         cgi_data = urllib.urlencode({"sid":self.SID,\
                                     "search":host,\
@@ -114,29 +116,25 @@ class CentreonServer(GenericServer):
                                     "o":"h",\
                                     "p":20102,\
                                     "time":0})
+                
+        result = self.FetchURL(self.monitor_cgi_url + "/include/monitoring/status/Hosts/" + self.XML_NDO + "/hostXML.php?"\
+                              + cgi_data, giveback="xml")        
+        xmlobj = result.result
         
-        result = self.FetchURL(self.nagios_cgi_url + "/include/monitoring/status/Hosts/xml/hostXML.php?"\
-                              + cgi_data, giveback="raw")
-        raw = result.result
-        # cut off <xml blabla>
-        xmlraw = ElementTree.fromstring(raw.split("\n")[1])
-        xmlobj = Actions.ObjectifyXML(xmlraw)   
-        del raw, xmlraw
-
         if len(xmlobj) != 0:
             # when connection by DNS is not configured do it by IP
             try:
-                if str(self.conf.connect_by_dns_yes) == "True":
+                if str(self.conf.connect_by_dns) == "True":
                    # try to get DNS name for ip, if not available use ip
                     try:
-                        address = socket.gethostbyaddr(xmlobj[0].a.text)[0]
+                        address = socket.gethostbyaddr(xmlobj.l.a.text)[0]
                         del xmlobj
                     except:
                         self.Error(sys.exc_info())
-                        address = str(xmlobj[0].a.text)                        
+                        address = str(xmlobj.l.a.text)                        
                         del xmlobj
                 else:
-                    address = str(xmlobj[0].a.text)
+                    address = str(xmlobj.l.a.text)
                     del xmlobj
             except:
                 result, error = self.Error(sys.exc_info())
@@ -158,8 +156,10 @@ class CentreonServer(GenericServer):
         gets a shiny new SID for XML HTTP requests to Centreon cutting it out via .partition() from raw HTML
         additionally get php session cookie
         """
+        login_data = urllib.urlencode({"useralias" : self.username, "password" : self.password, "submit" : "Login"})
+      
         try:
-            raw = self.FetchURL(self.nagios_cgi_url + "/index.php?" + urllib.urlencode({"p":1, "autologin":1, "useralias":self.MD5_username, "password":self.MD5_password}), giveback="raw")
+            raw = self.FetchURL(self.monitor_cgi_url + "/index.php",cgi_data=login_data, giveback="raw")
             del raw
             sid = str(self.Cookie._cookies.values()[0].values()[0]["PHPSESSID"].value)
             return Result(result=sid)
@@ -167,31 +167,30 @@ class CentreonServer(GenericServer):
             result, error = self.Error(sys.exc_info())
             return Result(result=result, error=error)
         
+    
+    def _get_ndo_url(self):
+        """
+        find out if Centreon is that new so it already supports ..../xml/ndo/.... URLs
+        """
+        ndo_test_url = self.monitor_cgi_url + "/include/monitoring/status/Hosts/xml/ndo/hostXML.php?" + urllib.urlencode({"num":0, "limit":999, "o":"hpb", "sort_type":"status", "sid":self.SID})
+        if "HTTP Error 404" in self.FetchURL(ndo_test_url).error:   
+            self.XML_NDO = "xml"
+            if str(self.conf.debug_mode) == "True":
+                self.Debug(server=self.get_name(), debug = "Centreon Monitor does not use .../xml/ndo/... URLs.")
+        
         
     def _get_host_id(self, host):
         """
         get host_id via parsing raw html
         """
-        cgi_data = urllib.urlencode({"p":201, "autologin":1,\
-                                    "o":"hd", "host_name":host,\
-                                    "useralias":self.MD5_username,\
-                                    "password":self.MD5_password})
-        result = self.FetchURL(self.nagios_cgi_url + "/main.php?" + cgi_data, giveback="raw")
+        cgi_data = urllib.urlencode({"p":201,\
+                                    "o":"hd", "host_name":host})
+        result = self.FetchURL(self.monitor_cgi_url + "/main.php?" + cgi_data, cgi_data=urllib.urlencode({"sid":self.SID}), giveback="raw")
         raw, error = result.result, result.error
 
         if error == "":
             host_id = raw.partition("var host_id = '")[2].partition("'")[0]
             del raw
-            # if for some reason host_id could not be retrieved because
-            # we get a login page clear cookies and SID and try again
-            if host_id == "":
-                if str(self.conf.debug_mode) == "True":
-                    self.Debug(server=self.get_name(), debug = "Host ID could not be retrieved, trying again...")
-                self.SID = self._get_sid().result
-                result = self.FetchURL(self.nagios_cgi_url + "/main.php?" + cgi_data, giveback="raw")
-                raw, error = result.result, result.error
-                host_id = raw.partition("var host_id= '")[2].partition("'")[0]
-                del raw
         else:
             if str(self.conf.debug_mode) == "True":
                 self.Debug(server=self.get_name(), debug = "Host ID could not be retrieved.")
@@ -215,25 +214,19 @@ class CentreonServer(GenericServer):
         """
         parse a ton of html to get a host and a service id...
         """
-        cgi_data = urllib.urlencode({"p":"20305",\
+        cgi_data = urllib.urlencode({"p":"20218",\
                                      "host_name":host,\
                                      "service_description":service,\
                                      "o":"as"})
-        result = self.FetchURL(self.nagios_cgi_url + "/main.php?"+ cgi_data, giveback="raw")
+        # might look strange to have cgi_data 2 times, the first it is the "real" in URL and the second is the cgi_data parameter
+        # from urllib to get the session id POSTed
+        result = self.FetchURL(self.monitor_cgi_url + "/main.php?"+ cgi_data, cgi_data=urllib.urlencode({"sid":self.SID}), giveback="raw")
         raw, error = result.result, result.error
         
-        # ids to give back, should contain to items, a host and a service id
+        # ids to give back, should contain two items, a host and a service id
         ids = []
         
         if error == "":
-            if raw.find('selected="selected"') == -1:
-                # looks there was this old SID problem again - get a new one 
-                if str(self.conf.debug_mode) == "True":
-                    self.Debug(server=self.get_name(), host=host, service=service, debug = "IDs could not be retrieved, trying again...")                
-                self.SID = self._get_sid().result
-                result = self.FetchURL(self.nagios_cgi_url + "/main.php?"+ cgi_data, giveback="raw")
-                raw, error = result.result, result.error
-                
             # search ids
             for l in raw.splitlines():
                 if l.find('selected="selected"') <> -1:
@@ -258,51 +251,42 @@ class CentreonServer(GenericServer):
         
         # get sid in case this has not yet been done
         if self.SID == None or self.SID == "":
-            self.SID = self._get_sid().result     
+            self.SID = self._get_sid().result
+            # those ndo urls would not be changing too often so this check migth be done here
+            self._get_ndo_url()
             
         # services (unknown, warning or critical?)
-        nagcgiurl_services = self.nagios_cgi_url + "/include/monitoring/status/Services/xml/serviceXML.php?" + urllib.urlencode({"num":0, "limit":999, "o":"svcpb", "sort_type":"status", "sid":self.SID})
+        nagcgiurl_services = self.monitor_cgi_url + "/include/monitoring/status/Services/" + self.XML_NDO + "/serviceXML.php?" + urllib.urlencode({"num":0, "limit":999, "o":"svcpb", "sort_type":"status", "sid":self.SID})
 
         # hosts (up or down or unreachable)
-        nagcgiurl_hosts = self.nagios_cgi_url + "/include/monitoring/status/Hosts/xml/hostXML.php?" + urllib.urlencode({"num":0, "limit":999, "o":"hpb", "sort_type":"status", "sid":self.SID})
+        nagcgiurl_hosts = self.monitor_cgi_url + "/include/monitoring/status/Hosts/" + self.XML_NDO + "/hostXML.php?" + urllib.urlencode({"num":0, "limit":999, "o":"hpb", "sort_type":"status", "sid":self.SID})
 
         # hosts - mostly the down ones
         # unfortunately the hosts status page has a different structure so
         # hosts must be analyzed separately
         try:           
-            result = self.FetchURL(nagcgiurl_hosts, giveback="raw")
-            raw, error = result.result, result.error
-            if error != "": return Result(result=raw, error=error)
-            xmlobj = list()
-            # cut off <xml blabla>
-            try:
-                xmlraw = ElementTree.fromstring(raw.split("\n")[1])
-                xmlobj = Actions.ObjectifyXML(xmlraw)            
-                del raw, error
-            except:
-                # in case we got nothing back
-                xmlraw = ""
-                self.Error(sys.exc_info())
-                       
+            result = self.FetchURL(nagcgiurl_hosts, giveback="xml") 
+            xmlobj, error = result.result, result.error            
+            if error != "": return Result(result=xmlobj, error=error)
+            
             # in case there are no children session id is invalid
-            if xmlraw == "<response>bad session id</response>":    
-                del raw, xmlraw
+            if xmlobj == "<response>bad session id</response>" or str(xmlobj) == "Bad Session ID":    
+                del xmlobj
                 if str(self.conf.debug_mode) == "True": 
                     self.Debug(server=self.get_name(), debug="Bad session ID, retrieving new one...")                
 
                 # try again...
                 self.SID = self._get_sid().result
-                result = self.FetchURL(nagcgiurl_hosts, giveback="raw")
-                raw, error = result.result, result.error
-                if error != "": return Result(result=raw, error=error)
-                # cut off <xml blabla>
-                xmlraw = ElementTree.fromstring(raw.split("\n")[1])
-                xmlobj = Actions.ObjectifyXML(xmlraw)
-                del raw, xmlraw, error
-            else:
-                del xmlraw
+                result = self.FetchURL(nagcgiurl_hosts, giveback="xml")
+                xmlobj, error = result.result, result.error
+                if error != "": return Result(result=xmlobj, error=error)
                 
-            for l in xmlobj:
+                # a second time a bad session id should raise an error                
+                if xmlobj == "<response>bad session id</response>" or str(xmlobj) == "Bad Session ID":
+                    #return Result(result=xmlobj, error=str(xmlobj))
+                    return Result(result="ERROR", error=str(xmlobj))
+                
+            for l in xmlobj.findAll("l"):
                 try:                       
                     n = {}
                     # host
@@ -316,7 +300,7 @@ class CentreonServer(GenericServer):
                     # status_information
                     n["status_information"] = str(l.ou.text)
                     # attempts are not shown in case of hosts so it defaults to "N/A"
-                    n["attempt"] = str(l.tr.text)
+                    n["attempt"] = str(l.tr.text).split(" ")[0]
                     # host acknowledged or not, has to be filtered
                     n["acknowledged"] = str(l.ha.text)
                     # host notification disabled or not, has to be filtered
@@ -324,41 +308,40 @@ class CentreonServer(GenericServer):
                     # host check enabled or not, has to be filtered
                     n["check_enabled"] = str(l.ace.text)
                     # host down for maintenance or not, has to be filtered
-                    n["in_downtime"] = str(l.hdtm.text)
-                                       
-                    # store information about acknowledged and down hosts for further reference
-                    if n["in_downtime"] == "1" : self.new_hosts_in_maintenance.append(n["host"])
-                    if n["acknowledged"] == "1" : self.new_hosts_acknowledged.append(n["host"])
-                     
-                    # what works in cgi-Nagios via cgi request has to be filtered out here "manually"
-                    if not (str(self.conf.filter_acknowledged_hosts_services) == "True" and \
-                       n["acknowledged"] == "1") and \
-                       not (str(self.conf.filter_hosts_services_disabled_notifications) == "True" and \
-                       n["notification_enabled"] == "0") and \
-                       not (str(self.conf.filter_hosts_services_disabled_checks) == "True" and \
-                       n["check_enabled"] == "0") and \
-                       not (str(self.conf.filter_hosts_services_maintenance) == "True" and \
-                       n["in_downtime"] == "1") and\
-                       not (str(self.conf.filter_all_down_hosts) == "True" and n["status"] == "DOWN") and\
-                       not (str(self.conf.filter_all_unreachable_hosts) == "True" and n["status"] == "UNREACHABLE"):
-                        # add dictionary full of information about this host item to nagitems
-                        nagitems["hosts"].append(n)
-                        # after collection data in nagitems create objects from its informations
-                        # host objects contain service objects
-                        if not self.new_hosts.has_key(n["host"]):
-                            new_host = n["host"]
-                            self.new_hosts[new_host] = GenericHost()
-                            self.new_hosts[new_host].name = n["host"]
-                            self.new_hosts[new_host].status = n["status"]
-                            self.new_hosts[new_host].last_check = n["last_check"]
-                            self.new_hosts[new_host].duration = n["duration"]
-                            self.new_hosts[new_host].attempt = n["attempt"]
-                            self.new_hosts[new_host].status_information= n["status_information"]
+                    n["scheduled_downtime"] = str(l.hdtm.text)
+                    # is host flapping?
+                    # the "is" flag indicates "is_flapping"... and doesn't seem to exist on hosts
+                    # because python whines when l.is.text is used we go the .find() way
+                    if l.find("is") != None:
+                        n["flapping"] = str(l.find("is").text)
+                    else:
+                        n["flapping"] = "0"
+                    # active checks enabled or passiveonly?
+                    n["passiveonly"] = str(l.ace.text)
+                    # add dictionary full of information about this host item to nagitems
+                    nagitems["hosts"].append(n)
+                    # after collection data in nagitems create objects from its informations
+                    # host objects contain service objects
+                    if not self.new_hosts.has_key(n["host"]):
+                        new_host = n["host"]
+                        self.new_hosts[new_host] = GenericHost()
+                        self.new_hosts[new_host].name = n["host"]
+                        self.new_hosts[new_host].status = n["status"]
+                        self.new_hosts[new_host].last_check = n["last_check"]
+                        self.new_hosts[new_host].duration = n["duration"]
+                        self.new_hosts[new_host].attempt = n["attempt"]
+                        self.new_hosts[new_host].status_information= n["status_information"]
+                        self.new_hosts[new_host].acknowledged = bool(int(n["acknowledged"]))
+                        self.new_hosts[new_host].scheduled_downtime = bool(int(n["scheduled_downtime"]))
+                        self.new_hosts[new_host].flapping = bool(int(n["flapping"]))
+                        self.new_hosts[new_host].notifications_disabled = not bool(int(n["notification_enabled"]))
+                        self.new_hosts[new_host].passiveonly = not bool(int(n["passiveonly"]))
                 except:
                     # set checking flag back to False
                     self.isChecking = False
                     #return self.Error(sys.exc_info())
                     result, error = self.Error(sys.exc_info())
+                    #return Result(result=result, error=error)
                     return Result(result=result, error=error)
             
             del xmlobj                
@@ -371,38 +354,23 @@ class CentreonServer(GenericServer):
 
         # services
         try:
-            result = self.FetchURL(nagcgiurl_services, giveback="raw")
-            raw, error = result.result, result.error
-            if error != "": return Result(result=raw, error=error)
-            xmlobj = list()
-            # cut off <xml blabla>
-            try:
-                xmlraw = ElementTree.fromstring(raw.split("\n")[1])
-                xmlobj = Actions.ObjectifyXML(xmlraw)            
-                del raw, error
-            except:
-                # in case we got nothing back
-                xmlraw = ""                
-                self.Error(sys.exc_info())
+            result = self.FetchURL(nagcgiurl_services, giveback="xml")            
+            xmlobj, error = result.result, result.error
+            if error != "": return Result(result=xmlobj, error=error)
             
             # in case there are no children session id is invalid
-            if xmlraw == "<response>bad session id</response>": 
+            if xmlobj == "<response>bad session id</response>" or xmlobj == "Bad Session ID": 
                 # debug
                 if str(self.conf.debug_mode) == "True": 
                     self.Debug(server=self.get_name(), debug="Bad session ID, retrieving new one...")                                
                 # try again...
                 self.SID = self._get_sid().result
-                result = self.FetchURL(nagcgiurl_services, giveback="raw")  
-                raw, error = result.result, result.error                
-                if error != "": return Result(result=raw, error=error)
-                # cut off <xml blabla>
-                xmlraw = ElementTree.fromstring(raw.split("\n")[1])
-                xmlobj = Actions.ObjectifyXML(xmlraw)
-                del raw, xmlraw, error
-            else:
-                del xmlraw                
-
-            for l in xmlobj:
+                result = self.FetchURL(nagcgiurl_services, giveback="xml")                             
+                xmlobj, error = result.result, result.error 
+                #if error != "": return Result(result=xmlobj, error=error)
+                if error != "": return Result(result="ERROR", error=error)
+                
+            for l in xmlobj.findAll("l"):
                 try:
                     n = {}
                     # host
@@ -420,50 +388,46 @@ class CentreonServer(GenericServer):
                     # duration
                     n["duration"] = str(l.d.text)
                     # attempt
-                    n["attempt"] = str(l.ca.text)
+                    n["attempt"] = str(l.ca.text).split(" ")[0]
                     # status_information
                     n["status_information"] = str(l.po.text)
                     # service is acknowledged or not, has to be filtered
                     n["acknowledged"] = str(l.pa.text)
                     # service notification enabled or not, has to be filtered
                     n["notification_enabled"] = str(l.ne.text)
-                    # service check enabled or not, has to be filtered
-                    n["check_enabled"] = str(l.ac.text)
+                    # active service check enabled or not, has to be filtered
+                    n["passiveonly"] = str(l.ac.text)
                     # service down for maintenance or not, has to be filtered
-                    n["in_downtime"] = str(l.dtm.text)
-
-                    # what works in cgi-Nagios via cgi request has to be filtered out here "manually"
-                    if not (str(self.conf.filter_acknowledged_hosts_services) == "True" and \
-                       n["acknowledged"] == "1") and \
-                       not (str(self.conf.filter_hosts_services_disabled_notifications) == "True" and \
-                       n["notification_enabled"] == "0") and \
-                       not (str(self.conf.filter_hosts_services_disabled_checks) == "True" and \
-                       n["check_enabled"] == "0") and \
-                       not (str(self.conf.filter_hosts_services_maintenance) == "True" and \
-                       n["in_downtime"] == "1") and\
-                       not (str(self.conf.filter_all_unknown_services) == "True" and n["status"] == "UNKNOWN") and\
-                       not (str(self.conf.filter_all_warning_services) == "True" and n["status"] == "WARNING") and\
-                       not (str(self.conf.filter_all_critical_services) == "True" and n["status"] == "CRITICAL"): 
-                        # add dictionary full of information about this service item to nagitems - only if service
-                        nagitems["services"].append(n)
-                        
-                        # after collection data in nagitems create objects of its informations
-                        # host objects contain service objects
-                        if not self.new_hosts.has_key(n["host"]):
-                            self.new_hosts[n["host"]] = GenericHost()
-                            self.new_hosts[n["host"]].name = n["host"]
-                            self.new_hosts[n["host"]].status = "UP"
-                        # if a service does not exist create its object
-                        if not self.new_hosts[n["host"]].services.has_key(n["service"]):
-                            new_service = n["service"]
-                            self.new_hosts[n["host"]].services[new_service] = GenericService()
-                            self.new_hosts[n["host"]].services[new_service].host = n["host"]
-                            self.new_hosts[n["host"]].services[new_service].name = n["service"]
-                            self.new_hosts[n["host"]].services[new_service].status = n["status"]
-                            self.new_hosts[n["host"]].services[new_service].last_check = n["last_check"]
-                            self.new_hosts[n["host"]].services[new_service].duration = n["duration"]
-                            self.new_hosts[n["host"]].services[new_service].attempt = n["attempt"]
-                            self.new_hosts[n["host"]].services[new_service].status_information = n["status_information"]
+                    n["scheduled_downtime"] = str(l.dtm.text)
+                    # is service flapping?
+                    # the "is" flag indicates "is_flapping"... and python whines when using l.is.text so we need to
+                    # use .find("is") instead
+                    n["flapping"] = str(l.find("is").text)
+                    # add dictionary full of information about this service item to nagitems - only if service
+                    nagitems["services"].append(n)
+                    
+                    # after collection data in nagitems create objects of its informations
+                    # host objects contain service objects
+                    if not self.new_hosts.has_key(n["host"]):
+                        self.new_hosts[n["host"]] = GenericHost()
+                        self.new_hosts[n["host"]].name = n["host"]
+                        self.new_hosts[n["host"]].status = "UP"
+                    # if a service does not exist create its object
+                    if not self.new_hosts[n["host"]].services.has_key(n["service"]):
+                        new_service = n["service"]
+                        self.new_hosts[n["host"]].services[new_service] = GenericService()
+                        self.new_hosts[n["host"]].services[new_service].host = n["host"]
+                        self.new_hosts[n["host"]].services[new_service].name = n["service"]
+                        self.new_hosts[n["host"]].services[new_service].status = n["status"]
+                        self.new_hosts[n["host"]].services[new_service].last_check = n["last_check"]
+                        self.new_hosts[n["host"]].services[new_service].duration = n["duration"]
+                        self.new_hosts[n["host"]].services[new_service].attempt = n["attempt"]
+                        self.new_hosts[n["host"]].services[new_service].status_information = n["status_information"]
+                        self.new_hosts[n["host"]].services[new_service].acknowledged = bool(int(n["acknowledged"]))
+                        self.new_hosts[n["host"]].services[new_service].scheduled_downtime = bool(int(n["scheduled_downtime"]))
+                        self.new_hosts[n["host"]].services[new_service].flapping = bool(int(n["flapping"]))
+                        self.new_hosts[n["host"]].services[new_service].notifications_disabled = not bool(int(n["notification_enabled"]))
+                        self.new_hosts[n["host"]].services[new_service].passiveonly = not bool(int(n["passiveonly"]))
                 except:
                     # set checking flag back to False
                     self.isChecking = False
@@ -496,11 +460,10 @@ class CentreonServer(GenericServer):
                         "persistent":int(persistent), "sticky":int(sticky), "ackhostservice":"0", "o":"hd", "en":"1"})
                 # debug
                 if str(self.conf.debug_mode) == "True": 
-                    #print self.get_name(), host +": " + self.nagios_cgi_url + "/main.php?"+ cgi_data     
-                    self.Debug(server=self.get_name(), host=host, debug=self.nagios_cgi_url + "/main.php?"+ cgi_data)                
+                    self.Debug(server=self.get_name(), host=host, debug=self.monitor_cgi_url + "/main.php?"+ cgi_data)                
 
                 # running remote cgi command, also possible with GET method     
-                raw = self.FetchURL(self.nagios_cgi_url + "/main.php?" + cgi_data, giveback="raw") 
+                raw = self.FetchURL(self.monitor_cgi_url + "/main.php?" + cgi_data, giveback="raw") 
                 del raw
                 
             # if host is acknowledged and all services should be to or if a service is acknowledged
@@ -523,11 +486,11 @@ class CentreonServer(GenericServer):
                             "sticky":int(sticky), "o":"svcd", "en":"1"}) 
                     # debug
                     if str(self.conf.debug_mode) == "True": 
-                        self.Debug(server=self.get_name(), host=host, service=s, debug=self.nagios_cgi_url + "/main.php?" + cgi_data)                
+                        self.Debug(server=self.get_name(), host=host, service=s, debug=self.monitor_cgi_url + "/main.php?" + cgi_data)                
 
                     # running remote cgi command with GET method, for some strange reason only working if
                     # giveback is "raw"
-                    raw = self.FetchURL(self.nagios_cgi_url + "/main.php?" + cgi_data, giveback="raw")
+                    raw = self.FetchURL(self.monitor_cgi_url + "/main.php?" + cgi_data, giveback="raw")
                     del raw
         except:
             self.Error(sys.exc_info())
@@ -546,7 +509,7 @@ class CentreonServer(GenericServer):
                 # fill and encode CGI data
                 cgi_data = urllib.urlencode({"cmd":"host_schedule_check", "actiontype":1,\
                                              "host_id":host_id, "sid":self.SID})
-                url = self.nagios_cgi_url + "/include/monitoring/objectDetails/xml/hostSendCommand.php?" + cgi_data
+                url = self.monitor_cgi_url + "/include/monitoring/objectDetails/xml/hostSendCommand.php?" + cgi_data
                 del host_id
             else:
                 # service @ host
@@ -554,7 +517,7 @@ class CentreonServer(GenericServer):
                 # fill and encode CGI data
                 cgi_data = urllib.urlencode({"cmd":"service_schedule_check", "actiontype":1,\
                                              "host_id":host_id, "service_id":service_id, "sid":self.SID})
-                url = self.nagios_cgi_url + "/include/monitoring/objectDetails/xml/serviceSendCommand.php?" + cgi_data
+                url = self.monitor_cgi_url + "/include/monitoring/objectDetails/xml/serviceSendCommand.php?" + cgi_data
                 del host_id, service_id
             # execute POST request
             raw = self.FetchURL(url, giveback="raw")
@@ -571,8 +534,9 @@ class CentreonServer(GenericServer):
             if service == "":
                 # host
                 host_id = self._get_host_id(host)
-                cgi_data = urllib.urlencode({"p":"20305",\
+                cgi_data = urllib.urlencode({"p":"20106",\
                                              "host_id":host_id,\
+                                             "host_or_hg[host_or_hg]":1,\
                                              "submitA":"Save",\
                                              "persistent":int(fixed),\
                                              "persistant":int(fixed),\
@@ -582,12 +546,12 @@ class CentreonServer(GenericServer):
                                              "o":"ah"})
                 # debug
                 if str(self.conf.debug_mode) == "True": 
-                    self.Debug(server=self.get_name(), host=host, debug=self.nagios_cgi_url + "/main.php?" + cgi_data)                
+                    self.Debug(server=self.get_name(), host=host, debug=self.monitor_cgi_url + "/main.php?" + cgi_data)                
 
             else:
                 # service
                 host_id, service_id = self._get_host_and_service_id(host, service)
-                cgi_data = urllib.urlencode({"p":"20305",\
+                cgi_data = urllib.urlencode({"p":"20218",\
                                              "host_id":host_id,\
                                              "service_id":service_id,\
                                              "submitA":"Save",\
@@ -598,10 +562,10 @@ class CentreonServer(GenericServer):
                                              "o":"as"})
                 # debug
                 if str(self.conf.debug_mode) == "True": 
-                    self.Debug(server=self.get_name(), host=host, service=service, debug=self.nagios_cgi_url + "/main.php?" + cgi_data)                
+                    self.Debug(server=self.get_name(), host=host, service=service, debug=self.monitor_cgi_url + "/main.php?" + cgi_data)                
 
             # running remote cgi command
-            raw = self.FetchURL(self.nagios_cgi_url + "/main.php", giveback="raw", cgi_data=cgi_data)   
+            raw = self.FetchURL(self.monitor_cgi_url + "/main.php", giveback="raw", cgi_data=cgi_data)   
             del raw
         except:
             self.Error(sys.exc_info())
@@ -613,6 +577,7 @@ class CentreonServer(GenericServer):
         """
         # a SIDcount of 300 should make 15 min when being run every 3 secs as it is at 
         # the moment in Actions.RefreshLoopOneServer()
+        # maybe this is unnecessary now that we authenticate via login/password, no md5
         if self.SIDcount >= 300:
             if str(self.conf.debug_mode) == "True":
                 self.Debug(server=self.get_name(), debug="Old SID: " + self.SID + " " + str(self.Cookie))                
