@@ -12,6 +12,7 @@ import time
 import traceback
 import base64
 import re
+import gobject
 
 # to let Linux distributions use their own BeautifulSoup if existent try importing local BeautifulSoup first
 # see https://sourceforge.net/tracker/?func=detail&atid=1101370&aid=3302612&group_id=236865
@@ -63,10 +64,16 @@ class GenericServer(object):
                        "flapping.gif" : "flapping"}
 
     # Entries for monitor default actions in context menu
-    MENU_ACTIONS = ["Recheck", "Acknowledge", "Submit check result", "Downtime"]
+    MENU_ACTIONS = ["Monitor", "Recheck", "Acknowledge", "Submit check result", "Downtime"]
     
     # Arguments available for submitting check results 
     SUBMIT_CHECK_RESULT_ARGS = ["check_output", "performance_data"]
+    
+    # URLs for browser shortlinks/buttons on popup window
+    BROWSER_URLS= { "monitor": "$MONITOR$",\
+                    "hosts": "$MONITOR-CGI$/status.cgi?hostgroup=all&style=hostdetail&hoststatustypes=12",\
+                    "services": "$MONITOR-CGI$/status.cgi?host=all&servicestatustypes=253",\
+                    "history": "$MONITOR-CGI$/history.cgi?host=all"}
    
     
     def __init__(self, **kwds):
@@ -74,8 +81,8 @@ class GenericServer(object):
         for k in kwds: self.__dict__[k] = kwds[k]
         
         self.type = ""
-        self.nagios_url = ""
-        self.nagios_cgi_url = ""
+        self.monitor_url = ""
+        self.monitor_cgi_url = ""
         self.username = ""
         self.password = ""
         self.use_proxy = False
@@ -85,9 +92,8 @@ class GenericServer(object):
         self.proxy_password = ""        
         self.hosts = dict()
         self.new_hosts = dict()
-        self.thread = ""
+        self.thread = None
         self.isChecking = False
-        self.debug = False
         self.CheckingForNewVersion = False
         self.WorstStatus = "UP"
         self.States = ["UP", "UNKNOWN", "WARNING", "CRITICAL", "UNREACHABLE", "DOWN"]
@@ -119,7 +125,9 @@ class GenericServer(object):
         self.TreeViewColumns = list()
         self.ListStore = None
         self.ListStoreColumns = list()
-        
+        # flag which decides if authentication has to be renewed
+        self.refresh_authentication = False
+
     
     def init_HTTP(self):
         """
@@ -131,6 +139,13 @@ class GenericServer(object):
                 self.HTTPheaders[giveback] = {"Authorization": "Basic " + base64.b64encode(self.get_username() + ":" + self.get_password())}
 
                 
+    def reset_HTTP(self):
+        """
+        if authentication fails try to reset any HTTP session stuff - might be different for different monitors
+        """
+        self.HTTPheaders = {}     
+        
+                
     def get_name(self):
         """
         return stringified name
@@ -154,11 +169,11 @@ class GenericServer(object):
 
     @classmethod
     def get_columns(cls, row):
-        """ Gets columns filled with row data """
+        """ Gets columns filled with row data """  
         for column_class in cls.COLUMNS:
             # str() necessary because MacOSX Python cries otherwise
             yield str(column_class(row))        
-        
+
         
     def set_recheck(self, thread_obj):
         self._set_recheck(thread_obj.host, thread_obj.service)
@@ -170,7 +185,7 @@ class GenericServer(object):
                 # Do not check passive only checks
                 return
         # get start time from Nagios as HTML to use same timezone setting like the locally installed Nagios
-        result = self.FetchURL(self.nagios_cgi_url + "/cmd.cgi?" + urllib.urlencode({"cmd_typ":"96", "host":host}), giveback="raw")
+        result = self.FetchURL(self.monitor_cgi_url + "/cmd.cgi?" + urllib.urlencode({"cmd_typ":"96", "host":host}), giveback="raw")
         html = result.result
         self.start_time = html.split("NAME='start_time' VALUE='")[1].split("'></b></td></tr>")[0]          
             
@@ -190,7 +205,7 @@ class GenericServer(object):
                                      ("force_check", "on"),\
                                      ("btnSubmit", "Commit")])
         # execute POST request
-        self.FetchURL(self.nagios_cgi_url + "/cmd.cgi", giveback="raw", cgi_data=cgi_data)
+        self.FetchURL(self.monitor_cgi_url + "/cmd.cgi", giveback="raw", cgi_data=cgi_data)
     
         
     def set_acknowledge(self, thread_obj):
@@ -203,7 +218,7 @@ class GenericServer(object):
      
         
     def _set_acknowledge(self, host, service, author, comment, sticky, notify, persistent, all_services=[]):
-        url = self.nagios_cgi_url + "/cmd.cgi"      
+        url = self.monitor_cgi_url + "/cmd.cgi"      
         
         # decision about host or service - they have different URLs
         # do not care about the doube %s (%s%s) - its ok, "flags" cares about the necessary "&"
@@ -226,8 +241,8 @@ class GenericServer(object):
                 sticky_ack = "&sticky_ack=on"
             else:
                 sticky_ack = ""
-            cgi_data = urllib.urlencode({"cmd_typ":"33", "cmd_mod":"2", "host":host, "com_author":author,\
-                                         "com_data":comment, "btnSubmit":"Commit"})\
+            cgi_data = urllib.urlencode([("cmd_typ","33"), ("cmd_mod","2"), ("host",host), ("com_author",author),\
+                                         ("com_data",comment), ("btnSubmit","Commit")])\
                                          + send_notification + persistent_comment + sticky_ack
             self.FetchURL(url, giveback="raw", cgi_data=cgi_data) 
             
@@ -250,8 +265,11 @@ class GenericServer(object):
                 sticky_ack = "&sticky_ack=on"
             else:
                 sticky_ack = ""
-            cgi_data = urllib.urlencode({"cmd_typ":"34", "cmd_mod":"2", "host":host, "service":service,\
-                                         "com_author":author, "com_data":comment, "btnSubmit":"Commit"})\
+            # for whatever silly reason Icinga depends on the correct order of submitted form items...
+            # see sf.net bug 3428844
+            # so whe cannot use a dictionary with urllib but a tuple full of tuples
+            cgi_data = urllib.urlencode([("cmd_typ","34"), ("cmd_mod","2"), ("host",host), ("service",service),\
+                                         ("com_author",author), ("com_data",comment), ("btnSubmit","Commit")])\
                                          + send_notification + persistent_comment + sticky_ack     
             # running remote cgi command        
             self.FetchURL(url, giveback="raw", cgi_data=cgi_data) 
@@ -265,22 +283,22 @@ class GenericServer(object):
                     send_notification = "&send_notification=on"
                 else:
                     send_notification = "" 
-            # dito for persistence...
-            if persistent == True:
-                persistent_comment = "&persistent=on"
-            else:
-                persistent_comment = ""
-            # ...and sticky acks too?
-            if sticky == True:
-                sticky_ack = "&sticky_ack=on"
-            else:
-                sticky_ack = ""
-                cgi_data = urllib.urlencode({"cmd_typ":"34", "cmd_mod":"2", "host":host, "service":s,\
-                                             "com_author":author, "com_data":comment, "btnSubmit":"Commit"})\
-                                             + send_notification + persistent_comment + sticky_ack  
-                #running remote cgi command        
-                self.FetchURL(url, giveback="raw", cgi_data=cgi_data)
-            
+                # dito for persistence...
+                if persistent == True:
+                    persistent_comment = "&persistent=on"
+                else:
+                    persistent_comment = ""
+                # ...and sticky acks too?
+                if sticky == True:
+                    sticky_ack = "&sticky_ack=on"
+                else:
+                    sticky_ack = ""
+                    cgi_data = urllib.urlencode([("cmd_typ","34"), ("cmd_mod","2"), ("host",host), ("service",s),\
+                                                 ("com_author",author), ("com_data",comment), ("btnSubmit","Commit")])\
+                                                 + send_notification + persistent_comment + sticky_ack  
+                    #running remote cgi command        
+                    self.FetchURL(url, giveback="raw", cgi_data=cgi_data)
+                
     
     def set_downtime(self, thread_obj):
         self._set_downtime(thread_obj.host, thread_obj.service, thread_obj.author, thread_obj.comment, thread_obj.fixed,
@@ -313,7 +331,7 @@ class GenericServer(object):
                                      ("minutes", minutes),\
                                      ("btnSubmit","Commit")])        
         # running remote cgi command
-        result = self.FetchURL(self.nagios_cgi_url + "/cmd.cgi", giveback="raw", cgi_data=cgi_data)
+        result = self.FetchURL(self.monitor_cgi_url + "/cmd.cgi", giveback="raw", cgi_data=cgi_data)
         raw = result.result
         
     
@@ -326,21 +344,22 @@ class GenericServer(object):
         """
         worker for submitting check result
         """
-        url = self.nagios_cgi_url + "/cmd.cgi"      
+        url = self.monitor_cgi_url + "/cmd.cgi"      
         
         # decision about host or service - they have different URLs
         if service == "":
             # host
-            cgi_data = urllib.urlencode({"cmd_typ":"87", "cmd_mod":"2", "host":host,\
-                                         "plugin_state":{"up":"0", "down":"1", "unreachable":"2"}[state], "plugin_output":check_output,\
-                                         "performance_data":performance_data, "btnSubmit":"Commit"})  
+            cgi_data = urllib.urlencode([("cmd_typ","87"), ("cmd_mod","2"), ("host",host),\
+                                         ("plugin_state",{"up":"0", "down":"1", "unreachable":"2"}[state]),\
+                                         ("plugin_output",check_output),\
+                                         ("performance_data",performance_data), ("btnSubmit","Commit")])  
             self.FetchURL(url, giveback="raw", cgi_data=cgi_data) 
             
         if service != "":
             # service @ host
-            cgi_data = urllib.urlencode({"cmd_typ":"30", "cmd_mod":"2", "host":host, "service":service,\
-                                         "plugin_state":{"ok":"0", "warning":"1", "critical":"2", "unknown":"3"}[state], "plugin_output":check_output,\
-                                         "performance_data":performance_data, "btnSubmit":"Commit"})          
+            cgi_data = urllib.urlencode([("cmd_typ","30"), ("cmd_mod","2"), ("host",host), ("service",service),\
+                                         ("plugin_state",{"ok":"0", "warning":"1", "critical":"2", "unknown":"3"}[state]), ("plugin_output",check_output),\
+                                         ("performance_data",performance_data), ("btnSubmit","Commit")])          
             # running remote cgi command        
             self.FetchURL(url, giveback="raw", cgi_data=cgi_data) 
 
@@ -351,7 +370,7 @@ class GenericServer(object):
         directly from web interface
         """
         try:
-            result = self.FetchURL(self.nagios_cgi_url + "/cmd.cgi?" + urllib.urlencode({"cmd_typ":"55", "host":host}))
+            result = self.FetchURL(self.monitor_cgi_url + "/cmd.cgi?" + urllib.urlencode({"cmd_typ":"55", "host":host}))
             html = result.result
             start_time = html.find(attrs={"name":"start_time"}).attrMap["value"]
             end_time = html.find(attrs={"name":"end_time"}).attrMap["value"]            
@@ -372,30 +391,26 @@ class GenericServer(object):
         else:
             typ = 2      
         if str(self.conf.debug_mode) == "True":
-            self.Debug(server=self.get_name(), host=host, service=service, debug="Open host/service monitor web page " + self.nagios_cgi_url + '/extinfo.cgi?' + urllib.urlencode({"type":typ, "host":host, "service":service}))
-        webbrowser.open(self.nagios_cgi_url + '/extinfo.cgi?' + urllib.urlencode({"type":typ, "host":host, "service":service}))
+            self.Debug(server=self.get_name(), host=host, service=service, debug="Open host/service monitor web page " + self.monitor_cgi_url + '/extinfo.cgi?' + urllib.urlencode({"type":typ, "host":host, "service":service}))        
+        webbrowser.open(self.monitor_cgi_url + '/extinfo.cgi?' + urllib.urlencode({"type":typ, "host":host, "service":service}))
 
         
-    def open_nagios(self):
-        webbrowser.open(self.nagios_url)
-        # debug
-        if str(self.conf.debug_mode) == "True":
-            self.Debug(server=self.get_name(), debug="Open monitor web page " + self.nagios_url)
+    def OpenBrowser(self, widget=None, url_type="", output=None):
+        """
+        multiple purpose open browser method for all open-a-browser-needs
+        """                
 
+        # first close popwin
+        if output <> None:
+            output.popwin.Close()
         
-    def open_services(self):
-        webbrowser.open(self.nagios_cgi_url + "/status.cgi?host=all&servicestatustypes=253")
-        # debug
-        if str(self.conf.debug_mode) == "True":
-            self.Debug(server=self.get_name(), debug="Open services web page " + self.nagios_url + "/status.cgi?host=all&servicestatustypes=253")
-        
-            
-    def open_hosts(self):
-        webbrowser.open(self.nagios_cgi_url + "/status.cgi?hostgroup=all&style=hostdetail&hoststatustypes=12")
-        # debug
-        if str(self.conf.debug_mode) == "True":
-            self.Debug(server=self.get_name(), debug="Open hosts web page " + self.nagios_url + "/status.cgi?hostgroup=all&style=hostdetail&hoststatustypes=12")
-
+        # run thread with action
+        action = Actions.Action(string=self.BROWSER_URLS[url_type],\
+                        type="browser",\
+                        conf=self.conf,\
+                        server=self)
+        action.run()
+    
             
     def _get_status(self):
         """
@@ -419,9 +434,9 @@ class GenericServer(object):
         # group them together
         hostserviceprops = 0
         # services (unknown, warning or critical?)
-        nagcgiurl_services = self.nagios_cgi_url + "/status.cgi?host=all&servicestatustypes=" + str(servicestatustypes) + "&serviceprops=" + str(hostserviceprops)
+        nagcgiurl_services = self.monitor_cgi_url + "/status.cgi?host=all&servicestatustypes=" + str(servicestatustypes) + "&serviceprops=" + str(hostserviceprops)
         # hosts (up or down or unreachable)
-        nagcgiurl_hosts = self.nagios_cgi_url + "/status.cgi?hostgroup=all&style=hostdetail&hoststatustypes=" + str(hoststatustypes) + "&hostprops=" + str(hostserviceprops)
+        nagcgiurl_hosts = self.monitor_cgi_url + "/status.cgi?hostgroup=all&style=hostdetail&hoststatustypes=" + str(hoststatustypes) + "&hostprops=" + str(hostserviceprops)
         # hosts - mostly the down ones
         # unfortunately the hosts status page has a different structure so
         # hosts must be analyzed separately
@@ -472,7 +487,10 @@ class GenericServer(object):
                         if len(tds) < 7:
                             # the old Nagios table
                             # status_information
-                            n["status_information"] = str(tds[4].string)
+                            if len(tds[4](text=not_empty)) == 0:
+                                n["status_information"] = ""
+                            else:    
+                                n["status_information"] = str(tds[4].string).encode("utf-8")
                             # attempts are not shown in case of hosts so it defaults to "N/A"
                             n["attempt"] = "N/A"
                         else:
@@ -481,7 +499,10 @@ class GenericServer(object):
                             # to be stripped
                             n["attempt"] = str(tds[4].string).strip()
                             # status_information
-                            n["status_information"] = str(tds[5].string)
+                            if len(tds[5](text=not_empty)) == 0:
+                                n["status_information"] = ""
+                            else:    
+                                n["status_information"] = str(tds[5].string).encode("utf-8")
                             
                         # status flags 
                         n["passiveonly"] = False
@@ -512,7 +533,7 @@ class GenericServer(object):
                             self.new_hosts[new_host].last_check = n["last_check"]
                             self.new_hosts[new_host].duration = n["duration"]
                             self.new_hosts[new_host].attempt = n["attempt"]
-                            self.new_hosts[new_host].status_information= n["status_information"]
+                            self.new_hosts[new_host].status_information= n["status_information"].encode("utf-8")
                             self.new_hosts[new_host].passiveonly = n["passiveonly"]
                             self.new_hosts[new_host].notifications_disabled = n["notifications_disabled"]
                             self.new_hosts[new_host].flapping = n["flapping"]
@@ -582,7 +603,10 @@ class GenericServer(object):
                         # to be stripped
                         n["attempt"] = str(tds[5](text=not_empty)[0]).strip()
                         # status_information
-                        n["status_information"] = str(tds[6](text=not_empty)[0])
+                        if len(tds[6](text=not_empty)) == 0:
+                            n["status_information"] = ""
+                        else:    
+                            n["status_information"] = str(tds[6](text=not_empty)[0]).encode("utf-8")
                         # status flags 
                         n["passiveonly"] = False
                         n["notifications_disabled"] = False
@@ -618,8 +642,8 @@ class GenericServer(object):
                             # cleaning
                             del icons
                         # if a service does not exist create its object
-                        if not self.new_hosts[n["host"]].services.has_key(n["service"]):
-                            new_service = n["service"]
+                        if not self.new_hosts[n["host"]].services.has_key(n["service"]):                           
+                            new_service = n["service"]                            
                             self.new_hosts[n["host"]].services[new_service] = GenericService()
                             self.new_hosts[n["host"]].services[new_service].host = n["host"]
                             self.new_hosts[n["host"]].services[new_service].name = n["service"]
@@ -627,7 +651,7 @@ class GenericServer(object):
                             self.new_hosts[n["host"]].services[new_service].last_check = n["last_check"]
                             self.new_hosts[n["host"]].services[new_service].duration = n["duration"]
                             self.new_hosts[n["host"]].services[new_service].attempt = n["attempt"]
-                            self.new_hosts[n["host"]].services[new_service].status_information = n["status_information"]
+                            self.new_hosts[n["host"]].services[new_service].status_information = n["status_information"].encode("utf-8")
                             self.new_hosts[n["host"]].services[new_service].passiveonly = n["passiveonly"]
                             self.new_hosts[n["host"]].services[new_service].notifications_disabled = n["notifications_disabled"]
                             self.new_hosts[n["host"]].services[new_service].flapping = n["flapping"]
@@ -652,10 +676,11 @@ class GenericServer(object):
         return Result()
     
         
-    def GetStatus(self):
+    def GetStatus(self, output=None):
         """
         get nagios status information from nagcgiurl and give it back
         as dictionary
+        output parameter is needed in case authentication failed so that popwin might ask for credentials
         """
 
         # set checking flag to be sure only one thread cares about this server
@@ -672,9 +697,28 @@ class GenericServer(object):
         self.status, self.status_description = status.result, status.error     
 
         if status.error != "":
-            self.isChecking = False
-            return Result(result=self.status, error=self.status_description)
-        
+            # ask for password if authorization failed
+            if "HTTP Error 401" in status.error or \
+               "HTTP Error 403" in status.error or \
+               "Bad Session ID" in status.error:
+                if str(self.conf.servers[self.name].enabled) == "True":
+                    while status.error != "":
+                        # clean existent authentication
+                        self.reset_HTTP() 
+                        self.refresh_authentication = True
+                        # needed to get valid credentials
+                        gobject.idle_add(output.RefreshDisplayStatus)
+                        # take a break not to DOS the monitor...
+                        time.sleep(20)
+                        status = self._get_status()
+                        self.status, self.status_description = status.result, status.error  
+                        # if monitor has been disabled do not try to connect to it
+                        if str(self.conf.servers[self.name].enabled) == "False":
+                            break
+            else:
+                self.isChecking = False
+                return Result(result=self.status, error=self.status_description)
+            
         # this part has been before in GUI.RefreshDisplay() - wrong place, here it needs to be reset
         self.nagitems_filtered = {"services":{"CRITICAL":[], "WARNING":[], "UNKNOWN":[]}, "hosts":{"DOWN":[], "UNREACHABLE":[]}}      
 
@@ -710,6 +754,11 @@ class GenericServer(object):
                         self.Debug(server=self.get_name(), debug="Filter: DOWNTIME " + str(host.name))
                     host.visible = False
     
+                if host.flapping == True and str(self.conf.filter_all_flapping_hosts) == "True":
+                    if str(self.conf.debug_mode) == "True":
+                        self.Debug(server=self.get_name(), debug="Filter: FLAPPING HOST " + str(host.name))
+                    host.visible = False
+    
                 if HostIsFilteredOutByRE(host.name, self.conf) == True:
                     if str(self.conf.debug_mode) == "True":
                         self.Debug(server=self.get_name(), debug="Filter: REGEXP " + str(host.name))
@@ -720,7 +769,7 @@ class GenericServer(object):
                         self.Debug(server=self.get_name(), debug="Filter: REGEXP " + str(host.name))
                     host.visible = False 
     
-                # Finegrain for the specific State
+                # Finegrain for the specific state
                 if host.status == "DOWN":
                     if str(self.conf.filter_all_down_hosts) == "True":
                         if str(self.conf.debug_mode) == "True":
@@ -762,7 +811,12 @@ class GenericServer(object):
                     if str(self.conf.debug_mode) == "True":
                         self.Debug(server=self.get_name(), debug="Filter: DOWNTIME " + str(host.name) + ";" + str(service.name))
                     service.visible = False
-
+                
+                if service.flapping == True and str(self.conf.filter_all_flapping_services) == "True":
+                    if str(self.conf.debug_mode) == "True":
+                        self.Debug(server=self.get_name(), debug="Filter: FLAPPING SERVICE " + str(host.name) + ";" + str(service.name))
+                    service.visible = False
+                    
                 if host.scheduled_downtime == True and str(self.conf.filter_services_on_hosts_in_maintenance) == "True":
                     if str(self.conf.debug_mode) == "True":
                         self.Debug(server=self.get_name(), debug="Filter: Service on host in DOWNTIME " + str(host.name) + ";" + str(service.name))
@@ -880,7 +934,7 @@ class GenericServer(object):
             
         # copy of listed nagitems for next comparison
         self.nagitems_filtered_list = copy.copy(new_nagitems_filtered_list)
-
+        
         # put new informations into respective dictionaries      
         self.hosts = copy.copy(self.new_hosts)
         self.new_hosts.clear()
@@ -927,10 +981,7 @@ class GenericServer(object):
             
             # objectified HTML
             if giveback == 'obj':
-                #request = urllib2.Request(url, cgi_data, self.HTTPheaders['obj'])
-                # use opener - if cgi_data is not empty urllib uses a POST request
-                #urlcontent = self.urlopener.open(request)
-                yummysoup = BeautifulSoup(urlcontent, convertEntities=BeautifulSoup.ALL_ENTITIES)
+                yummysoup = BeautifulSoup(unicode(urlcontent.read(), "utf8", errors="ignore"), convertEntities=BeautifulSoup.ALL_ENTITIES)
                 urlcontent.close()                
                 del urlcontent
                 #return Result(result=copy.deepcopy(yummysoup))
@@ -938,8 +989,6 @@ class GenericServer(object):
 
             # objectified generic XML, valid at least for Opsview and Centreon
             elif giveback == "xml":
-                #request = urllib2.Request(url, cgi_data, self.HTTPheaders[giveback])
-                #urlcontent = self.urlopener.open(request)
                 xmlobj = BeautifulStoneSoup(urlcontent.read(), convertEntities=BeautifulStoneSoup.XML_ENTITIES)
                 urlcontent.close()
                 del urlcontent  
@@ -963,14 +1012,14 @@ class GenericServer(object):
         """
 
         # the fasted method is taking hostname as used in monitor
-        if str(self.conf.connect_by_host) == "True":
+        if str(self.conf.connect_by_host) == "True" or host == "":
             return Result(result=host)
         
         # initialize ip string
         ip = ""
 
         # glue nagios cgi url and hostinfo 
-        nagcgiurl_host  = self.nagios_cgi_url + "/extinfo.cgi?type=1&host=" + host
+        nagcgiurl_host  = self.monitor_cgi_url + "/extinfo.cgi?type=1&host=" + host
         
         # get host info
         result = self.FetchURL(nagcgiurl_host, giveback="obj")
@@ -1016,8 +1065,6 @@ class GenericServer(object):
         # do some garbage collection
         gc.collect()  
         
-        
-    
     
     def Error(self, error):
         """
@@ -1039,4 +1086,5 @@ class GenericServer(object):
         debug_string =  " ".join((head + ":",  str(datetime.datetime.now()), server, host, service, debug))     
         # give debug info to debug loop for thread-save log-file writing
         self.debug_queue.put(debug_string)
-        
\ No newline at end of file
+        
+    
